
# THIS FILE IS GENERATED FROM PYART SETUP.PY
short_version = '1.11.1'
version = '1.11.1'
full_version = '1.11.1'
git_revision = 'd216a2750da3014b3f387f3915b70730c64763cb'
release = True

if not release:
    version = full_version
